@@ -459,6 +459,7 @@ main (int argc, char **argv)
 
   gint newargc = 0;
   gchar **newargv = NULL;
+  gint8 exit_status = -1;
 
   guint run_mode = DEFAULT_MODE;
 
@@ -670,6 +671,24 @@ main (int argc, char **argv)
 
       for (i = optind + 1; i < newargc; i++)
 	{
+	   // in sudo mode, check for ' and \ in string and escape it
+	  if(run_mode == SUDO_MODE && 
+	     (strchr(newargv[i],'\'') || strchr(newargv[i],'\\'))) 
+	       {
+		  const gchar *p = newargv[i];
+		  gchar *q,*dest;
+		  dest = q = g_malloc(strlen(newargv[i])*2+1);
+		  while(*p) {
+		     if (*p == '\'')
+			*q++ = '\\';
+		     else if (*p == '\\')
+			*q++ = '\\';
+		     *q++ = *p++;
+		  }
+		  *q = 0;
+		  g_free(newargv[i]);
+		  newargv[i] = dest;
+	       }
 	  tmp = g_strconcat (command, " '", newargv[i], "'", NULL);
 	  g_free (command);
 	  command = tmp;
@@ -716,21 +735,24 @@ main (int argc, char **argv)
 	  }
 
 	if (run_mode == SUDO_MODE)
-	  gksu_sudo_full (context,
-			  NULL, NULL,
-			  NULL, NULL,
-			  &error);
+	  gksu_sudo_fuller (context,
+			    NULL, NULL,
+			    NULL, NULL,
+			    &exit_status,
+			    &error);
 	else if (run_mode == SU_MODE)
-	  gksu_su_full (context,
-			NULL, NULL,
-			NULL, NULL,
-			&error);
+	  gksu_su_fuller (context,
+			  NULL, NULL,
+			  NULL, NULL,
+			  &exit_status,
+			  &error);
 
 	else
-	  gksu_run_full (context,
-			 NULL, NULL,
-			 NULL, NULL,
-			 &error);
+	  gksu_run_fuller (context,
+			   NULL, NULL,
+			   NULL, NULL,
+			   &exit_status,
+			   &error);
 	if ((error == NULL) || (error->code != GKSU_ERROR_WRONGPASS))
 	  break;
       }
@@ -746,5 +768,5 @@ main (int argc, char **argv)
       return 3;
     }
 
-  return 0;
+  return exit_status;
 }
